#!/usr/bin/python2

###################################################################################################################
##### Script installs/updates the license file
#####
##### Syntax:  license-manager.py <deployment type> <temporary directory where license file exists>
##### Example: license-manager.py deploy /tmp/tempdirectory
##### 
##### External service is expected to copy the new License file under a temporary directory prior to invocation. 
###################################################################################################################

import os, shutil, logging, sys
from optparse import OptionParser

import subprocess
from subprocess import call , Popen, PIPE

logger = None
CONST = None
does_license_temporary_file_exist = False
temporary_license_location = None

############################################################
##### MODIFY THIS SECTION FOR LOCATION/CONFIGURATION CHANGES
############################################################
class CONST(object):
	license_file = "ECX.lic"
	license_location = "/opt/ECX/virgo/repository/ecx-usr"    

	def __setattr__(self, *_):
		pass

#############################################################

def main(argv):

	global logger    
	logger = logging.getLogger(__name__)
	logger.setLevel(logging.INFO)

	# create console handler and set level to info
	console = logging.StreamHandler()
	console.setLevel(logging.ERROR)
	consoleformatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
	console.setFormatter(consoleformatter)
	logger.addHandler(console)

	# create a file handler
	handler = logging.FileHandler(os.path.join('/data/log/adminconsole','license-manager.log'))
	handler.setLevel(logging.INFO)
	# create a logging format
	formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
	handler.setFormatter(formatter)
	# add the handlers to the logger
	logger.addHandler(handler)
    
	if len(sys.argv) == 3:
		add_wide_gaps()
		logger.info('##################################################################')
		logger.info('RUNNING LICENSE MANAGER')
		logger.info('Argument Length ' + str(len(argv)))
		logger.info('Arguments: ' + str(argv))
		process(argv[1], argv[2])
	else:
		logger.error('Invalid number of arguments')
		logger.error('Syntax:  license-manager.py <mode> <temporary_directory>')
		logger.error('Example: license-manager.py deploy /tmp/tempdirectory')        
		failed_exit()

def check_file_exists(file_path, type):
        logger.info('File location: ' + file_path)
        if (os.path.isfile(file_path)):
            logger.info('File exists')
            if type == "tempLicense":
                set_temporary_license_property()
        else:
            logger.info('License file does NOT exist')

def set_temporary_license_property():   
	global does_license_temporary_file_exist
	does_license_temporary_file_exist = True
            
def delete_file(file_path):
        if (os.path.isfile(file_path)):
            try:
                logger.info('Deleting ' + file_path)
                os.remove(file_path)
            except OSError as e:
                logger.error('Error: %s' % e)
                if e.errno != errno.ENOENT:
                    raise
                    failed_exit()

def delete_directory(directory):
    try:
        logger.info('Deleting directory ' + directory)
        shutil.rmtree(directory, ignore_errors=True)
    except shutil.Error as e:
        logger.error('Error: %s' % e)
        failed_exit()
    except IOError as e:
        logger.error('Error: %s' % e.strerror)
        failed_exit()

def copy_file(src, dest):
    try:
        logger.info('Copying ' + src + ' to ' + dest)
        shutil.copy2(src, dest)
    except shutil.Error as e:
        logger.error('Error: %s' % e)
        failed_exit()
    except IOError as e:
        logger.error('Error: %s' % e.strerror)
        failed_exit()

def verify_file(licensefile):
	cmd = '/usr/bin/file -b ' + licensefile
	p = Popen(cmd , shell=True, stdout=PIPE, stderr=PIPE)
	stdout, stderr = p.communicate()

	if p.returncode == 0: 
		if str(stdout).rstrip() != "data":
			logger.info('Invalid license file type ' + str(stdout))
	                delete_directory(temporary_license_location)
			failed_exit()
	else:
		logger.error('Failure in verifying license file (type)')
                delete_directory(temporary_license_location)
		failed_exit()


	cmd = '/usr/bin/file -ib ' + licensefile
	p = Popen(cmd , shell=True, stdout=PIPE, stderr=PIPE)
	stdout, stderr = p.communicate()

	if p.returncode == 0: 
		if str(stdout).rstrip() != "application/octet-stream; charset=binary":
			logger.info('Invalid license mime type ' + str(stdout))
                	delete_directory(temporary_license_location)
			failed_exit()
	else:
		logger.error('Failure in verifying license file (mime type)')
                delete_directory(temporary_license_location)
		failed_exit()

	logger.info('License file has been verified')

def process(mode,temporary_directory):
        if mode == "deploy":   
            global CONST
            CONST = CONST()
	    global temporary_license_location
	    temporary_license_location = temporary_directory
            # check license file exists under product location
            check_file_exists(os.path.join(CONST.license_location,CONST.license_file), "default")
            # check whether new license file exists under temporary location (copied by spring boot)
            check_file_exists(os.path.join(temporary_license_location,CONST.license_file), "tempLicense")
            if does_license_temporary_file_exist:
                verify_file(os.path.join(temporary_license_location, CONST.license_file))
		delete_file(os.path.join(CONST.license_location,CONST.license_file))
                copy_file(os.path.join(temporary_license_location,CONST.license_file), os.path.join(CONST.license_location,CONST.license_file))
                delete_directory(temporary_license_location)
		reload_license()
                clean_exit()
            else:
                logger.error("Cannot find " + CONST.license_file + " under " + temporary_license_location)
                failed_exit()
        else:
            logger.info('Unknown mode, please check usage')
            failed_exit()

def reload_license():
        logger.info('reload_license ==>')	
	cmd='/usr/bin/curl -H "Accept: application/json" -H "Content-Type: application/json" -s -X POST  http://localhost:8082/api/endeavour/session?action=installLicense'
	p = Popen( cmd , shell=True, stdout=PIPE, stderr=PIPE)
	stdout, stderr = p.communicate()

	logger.info('cmd: ' + cmd + ' rc: ' + str(p.returncode))
   
	if p.returncode == 0 : 
		logger.info('License reload was successful.')
	else:
		logger.error('License reload was NOT successful. You need to restart the application manually')

def add_wide_gaps():
        for num in range(1,5): 
            logger.info('')

def clean_exit():
	logger.info('')
	logger.info('Script completed successfully, Exiting!')
	logger.info('')
	sys.exit(0)

def failed_exit():
	logger.info('')
	logger.error('Script completed with failures')        
	sys.exit(1)

if __name__ == "__main__":
	main(sys.argv)
