#!/usr/bin/python2

###################################################################################################################
##### Script provides authentication related functionalities
#####
#####
##### Syntax:  login-manager.py --mode <mode> --username <username> [--password <password>]
#####
##### Example: login-manager.py --mode checklastpasswdchange --username testuser 
##### Example: login-manager.py --mode checkauth --username testuser --password testpassword
#####
###################################################################################################################


from subprocess import call ,Popen, PIPE
import time
import string
import random
import os, tempfile, shutil, logging, sys, re
from optparse import OptionParser, SUPPRESS_HELP 
import urllib
import re
import base64

CONST = None
FORWARD_SLASH = "/"
SERVER_ADMIN_USER = "serveradmin"

class LessThenFilter(logging.Filter):
    def __init__(self, level):
	self._level = level
	logging.Filter.__init__(self)

    def filter(self, rec):
        return rec.levelno < self._level

def setup():
	global logger
	logger = logging.getLogger('login_manager')
	logger.setLevel(logging.DEBUG)

	# create a console handler
	cHandler = logging.StreamHandler(sys.stdout)
	cHandler.setLevel(logging.INFO)
	cHandler.setFormatter(logging.Formatter('%(message)s'))
	cHandler.addFilter(LessThenFilter(logging.WARNING))
	logger.addHandler(cHandler)

	# create a console handler for stderr
	ceHandler = logging.StreamHandler()
	ceHandler.setLevel(logging.WARNING)
	ceHandler.setFormatter(logging.Formatter('%(message)s'))
	logger.addHandler(ceHandler)

	# create a file handler
	fHandler = logging.FileHandler(options.log)
	fHandler.setLevel(logging.DEBUG)
	fHandler.setFormatter(logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s'))
	logger.addHandler(fHandler)

	add_wide_gaps() 
	logger.debug('+--------------------------------------------------------------------------------+')
	logger.debug('+------------------------- STARTING LOGIN MANAGER -------------------------------+')
	logger.debug('')
	logger.debug('Current local time: ' + time.asctime(time.localtime(time.time())) ); 
	logger.debug('')
	logger.debug(os.path.realpath(__file__) + ' called with options: mode: ' + options.mode + ' username: ' + options.username)

def main():
	
	if options.mode == 'checklastpasswdchange':
		check_last_passwd_change(options.username)
	
	if options.mode == 'checkauth':
		check_authentication(options.username, options.password)

	if options.mode == 'changepassword':
		change_password(options.username, options.newpassword)

	failed_exit()

"""
Gets last password change information for a given username

@param username
"""
def check_last_passwd_change(username):
	logger.debug('Checking password change for user ' + username)
	command = '/usr/bin/chage -l ' + username +  ' | /usr/bin/grep \"Last password change\" | /usr/bin/awk -F: \'{print $2}\' | /usr/bin/awk \'$1=$1\'' 
	pobj = Popen(command, bufsize=1, shell=True, stdout=PIPE, stderr=PIPE, cwd='/tmp')
	out, err = pobj.communicate()
    
	logger.debug('Command Returncode : ' + str(pobj.returncode))

	process_command_output(pobj.returncode, out, err)

"""
Validates authentication for a given username and password

@param username
@param password
"""
def check_authentication(username, password):
	logger.info('Checking authentication for user ')
	command = '/bin/python3 '+  os.path.dirname(os.path.realpath(__file__)) +  '/checkauth.py '+ username + ' ' + password  

	
	pobj = Popen(command, bufsize=1, shell=True, stdout=PIPE, stderr=PIPE, cwd='/tmp')
	out, err = pobj.communicate()
    
	logger.debug('Command Returncode : ' + str(pobj.returncode))

	process_command_output(pobj.returncode, out, err)

	
"""
Change password for os administrator

@param username
@param newpassword
"""
def change_password(username, newpassword):
	user = base64.b64decode(username)

	if user == SERVER_ADMIN_USER:
		tmpFile = tempfile.NamedTemporaryFile(delete=False)
		try:
			#logger.info("file:"+tmpFile.name)
			tmpFile.write(user+":"+base64.b64decode(newpassword))
			tmpFile.flush()
			command = '/usr/sbin/chpasswd < '+ tmpFile.name
			print(command)
			pobj = Popen(command, bufsize=1, shell=True, stdout=PIPE, stderr=PIPE, cwd='/tmp')
			out, err = pobj.communicate()
    
			logger.debug('Command Returncode : ' + str(pobj.returncode))
		
			process_command_output(pobj.returncode, out, err)
		finally:
			os.unlink(tmpFile.name)
			tmpFile.close()

		
	else:
		logger.debug('Change password for ' + user + ' is not permitted')
		failed_exit()

def escapeSpecialChars(password):
	return re.sub(r'([\\\$])', r'\\\1', password)
	
def process_command_output(returncode, out, err):

	if returncode == 0:    
        	if out:
			logger.debug('')
            		logger.debug('Command Output :')
            		logger.debug('----------------')
            		logger.debug('')
            		logger.info(out)            
		
		clean_exit()
	else:
       		if out:
			logger.debug('')
			logger.debug('Command Output :')
			logger.debug('----------------')
			logger.debug('')
			logger.error(out)

		if err:
			logger.debug('')
			logger.debug('Command Errors :')
			logger.debug('----------------')
			logger.debug('')
			logger.error(err)                

		failed_exit()

def add_wide_gaps():
        for num in range(1,5): 
            logger.info('')

def clean_exit():
	logger.info('')
	logger.info('Script completed successfully, Exiting!')
	logger.info('')
	sys.exit(0)

def failed_exit():
	logger.info('')
	logger.error('Script completed with failures')        
	sys.exit(1)

if __name__ == "__main__":

	parser = OptionParser(version='%prog version 1.0')

	usage = "usage: %prog [options]"
	parser = OptionParser(usage=usage)
    
	parser.add_option("--version", help="version")

	parser.add_option("-l","--log",
                      default='/data/log/adminconsole/login-manager.log',
                      dest='log',
                      help="Log file location"
                      " [ default: %default ]" )

	parser.add_option("-v", action="count", dest="verbosity", help=SUPPRESS_HELP)

	parser.add_option("--mode",
                      help="Supported modes for login manager: checklastpasswdchange, checkauth")

	parser.add_option("--username",
                      help="Username for authentication check")

	parser.add_option("--password",
                      help="Password for the user from --username")

	parser.add_option("--newpassword",
                      help="New password for the user from --username")

	(options, args) = parser.parse_args()

	setup()

	if options.mode is None:
		logger.error('')
		logger.error('Mandatory option \'mode\' is missing')
		parser.print_help()
		failed_exit()

	if options.mode != 'checklastpasswdchange' and options.mode != 'checkauth' and options.mode != 'changepassword':
		logger.error('')
		logger.error('Invalid \'mode\' specified')
		parser.print_help()
		failed_exit()

	if options.mode == 'checklastpasswdchange' and not options.username:
		logger.error('')
		logger.error('Mode \'checklastpasswdchnage\' requires username option')
		parser.print_help()
		failed_exit()

	if options.mode == 'checkauth':
		
		if not options.username or options.username is None:
			logger.error('')
			logger.error('Mode \'checkauth\' requires username option and cannot be null')
			parser.print_help()
			failed_exit()
			
		if not options.password or options.password is None:
			logger.error('')
			logger.error('Mode \'checkauth\' requires password  option and cannot be null')
			parser.print_help()
			failed_exit()
		

	if options.mode == 'changepassword':

		if not options.username or options.username is None:
			logger.error('')
			logger.error('Mode \'changepassword\' requires username option and cannot be null')
			parser.print_help()
			failed_exit()

		if not options.newpassword or options.newpassword is None:
			logger.error('')
			logger.error('Mode \'changepasswd\' requires newpassword option and cannot be null')
			parser.print_help()
			failed_exit()

	main()
