#!/usr/bin/python2

####################################################################################################################
##### Script runs ECX udpates
#####
##### Syntax: plugin-manager.py --mode <deploy> --filename <plugin file name> --filedirectory <directory in which plugin file exists>
##### Syntax: plugin-manager.py --mode <delete> --filename <plugin file to be deleted>
##### 
##### 
####################################################################################################################

from subprocess import call ,Popen, PIPE
import string
import random
import os, shutil, logging, sys, re
from optparse import OptionParser, SUPPRESS_HELP 
import urllib
from time import sleep

CONST = None

class LessThenFilter(logging.Filter):
    def __init__(self, level):
	self._level = level
	logging.Filter.__init__(self)

    def filter(self, rec):
        return rec.levelno < self._level

############################################################
##### MODIFY THIS SECTION FOR LOCATION/CONFIGURATION CHANGES
############################################################
class CONST(object):
	virgo_pickup_directory = "/opt/virgo/pickup"

	def __setattr__(self, *_):
		pass
###########################################################

def main():

	log = logging.getLogger()
	log.setLevel(logging.NOTSET)	

	sh_out = logging.StreamHandler(sys.stdout)
	sh_out.setLevel(logging.INFO)
	sh_out.setFormatter(logging.Formatter('%(message)s'))
	sh_out.addFilter(LessThenFilter(logging.WARNING))
	log.addHandler(sh_out)

	sh_err = logging.StreamHandler(sys.stderr)
	sh_err.setLevel(logging.WARNING)
	sh_err.setFormatter(logging.Formatter('%(message)s'))
	log.addHandler(sh_err)

	# create a file handler
	fhandler = logging.FileHandler(options.log)
	fhandler.setLevel(logging.DEBUG)
	fhandler.setFormatter(logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s'))
	log.addHandler(fhandler)

        logging.debug('----------------')
        logging.debug( 'Call ' + os.path.realpath(__file__) + ' with options:' )
        logging.debug(options)
        
	global CONST
	CONST = CONST()
    
	if options.mode == 'deploy': deploy_plugin()

	if options.mode == 'delete': delete_plugin()

def check_file_exists(file_path):
	logging.debug('File location: ' + file_path)	
	if (os.path.isfile(file_path)):
		logging.debug('File exists')
		return True
	else:
		logging.debug('File does NOT exist')
		return False

def check_directory_exists(directory_path):
        logging.debug('Directory location: ' + directory_path)	
	if (os.path.isdir(directory_path)):
		logging.debug(directory_path + ' directory exists')
		return True
	else:
		logging.debug(directory_path + ' directory does NOT exist')
		return False

def add_wide_gaps():
        for num in range(1,5): 
            logging.debug('')

def clean_exit():
        logging.debug('')
        logging.debug('Script completed successfully, Exiting!')
        logging.debug('')
        sys.exit(0)

def failed_exit(exitcode=1):
        logging.debug('')
        logging.debug('Script completed with failures ......')
        sys.exit(exitcode)

def deploy_plugin():
	if (check_directory_exists(CONST.virgo_pickup_directory)):
		if (check_file_exists(os.path.join(CONST.virgo_pickup_directory,options.filename))):
			logging.debug('Plugin ' + options.filename + ' is already found to be deployed')
			delete_plugin()
			
		copy_file(os.path.join(options.filedirectory,options.filename),CONST.virgo_pickup_directory)
		sleep(5)
		if not (check_plugin_status()):
			logging.debug(options.filename + ' was deployed but we cannot determine if it was STARTED')
			delete_directory(options.filedirectory)
			failed_exit(3)
		else:
			delete_directory(options.filedirectory)

def delete_plugin():
	if (check_directory_exists(CONST.virgo_pickup_directory)):
		if(check_file_exists(os.path.join(CONST.virgo_pickup_directory,options.filename))):
			delete_file(os.path.join(CONST.virgo_pickup_directory,options.filename))

def check_plugin_status():
	
	logging.debug('Checking deployment status of the plugin: ' + options.filename)

	pluginJar = options.filename
	pluginName = pluginJar.replace(".jar","")
	pluginPackagesHyphenSplit = pluginName.split("-")
	pluginPackages = pluginPackagesHyphenSplit[0].split(".")
	if (len(pluginPackages) > 4):
		bundleName = pluginPackages[0] + '.' + pluginPackages[1] + '.' + pluginPackages[2] + '.' + pluginPackages[3] + '.' + pluginPackages[4] 
	elif (len(pluginPackages) == 4):
		bundleName = pluginPackages[0] + '.' + pluginPackages[1] + '.' +  pluginPackages[2] + '.' + pluginPackages[3] 
	elif (len(pluginPackages) == 3):
		bundleName = pluginPackages[0] + '.' + pluginPackages[1] + '.' + pluginPackages[2]
	elif (len(pluginPackages) == 2):
		bundleName = pluginPackages[0] + '.' + pluginPackages[1]
	elif (len(pluginPackages) == 1):
		bundleName = pluginPackages[0]
	else:
		bundleName = pluginName

	command = 'tail --lines=50 /opt/virgo/serviceability/logs/log.log | grep --line-buffered \'Bundle ' + bundleName + '.*, BundleEvent STARTED\''
    
	logging.debug('Command: ' + command)
    
	pobj = Popen(command, bufsize=1, shell=True, stdout=PIPE, stderr=PIPE, cwd='/tmp')
	out, err = pobj.communicate()
    
	logging.debug('Command Returncode : ' + str(pobj.returncode))

	if pobj.returncode == 0:    
        	if out:
			logging.debug('')
			logging.debug('Command Output :')
			logging.debug('----------------')
			logging.debug('')
			logging.info(out)

		return True
	else:
		if out:
			logging.debug('')
			logging.debug('Command Output :')
			logging.debug('----------------')
			logging.debug('')
			logging.info(out)

		if err:
			logging.debug('')
			logging.debug('Command Errors :')
			logging.debug('----------------')
			logging.debug('')
			logging.error(err)                

		return False

def copy_file(src, dest):
	try:	
		logging.debug('Copying ' + src + ' to ' + dest)
		shutil.copy2(src, dest)
	except shutil.Error as e:
		logging.error('Error: %s' % e)
		failed_exit()
	except IOError as e:
		logging.error('Error: %s' % e.strerror)
		failed_exit()

def delete_file(file_path):
        if (os.path.isfile(file_path)):
            try:
                logging.debug('Deleting ' + file_path)
                os.remove(file_path)
            except OSError as e:
                logging.error('Error: %s' % e)
                if e.errno != errno.ENOENT:
                    raise
                    failed_exit()

def delete_directory(directory):
    try:
        logging.debug('Deleting directory ' + directory)
        shutil.rmtree(directory, ignore_errors=True)
    except shutil.Error as e:
        logging.error('Error: %s' % e)
        failed_exit()
    except IOError as e:
        logging.error('Error: %s' % e.strerror)
        failed_exit()

if __name__ == "__main__":

	parser = OptionParser(version='%prog version 1.0')

	usage = "usage: %prog [options]"
	parser = OptionParser(usage=usage)
    
	parser.add_option("--version", help="version")

	parser.add_option("-l","--log",
                      default='/opt/ECX/ecxemi/plugin-manager.log',
                      dest='log',
                      help="Log file location"
                      " [ default: %default ]" )

	parser.add_option("-v", action="count", dest="verbosity", help=SUPPRESS_HELP)

	parser.add_option("--mode",
                      help="supported mode: deploy")

	parser.add_option("--filename",
                      help="name of the file to be uploaded")

	parser.add_option("--filedirectory",
                      help="directory where the file to be uploaded exists")

	(options, args) = parser.parse_args()

	if options.mode is None:
		print ("A mandatory option 'mode' is missing\n")
		parser.print_help()
		failed_exit()   
        else:
		if options.mode == 'deploy':	
			if options.filename is None:
				print ("A mandatory option 'filename' is missing\n")
				parser.print_help()
				failed_exit() 

			if options.filedirectory is None:
				print ("A mandatory option 'filedirectory' is missing\n")
				parser.print_help()
				failed_exit() 

		if options.mode == 'delete':
			if options.filename is None:
				print ("A mandatory option 'filename' is missing\n")
				parser.print_help()
				failed_exit() 

	main()
