"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var core_1 = require('@angular/core');
var dashboard_service_1 = require('../services/dashboard.service');
var emi_service_1 = require('../../emi.service');
var DashboardComponent = (function () {
    function DashboardComponent(dashboardService, emiService) {
        this.dashboardService = dashboardService;
        this.emiService = emiService;
        this.mode = 'Observable';
        this.rpmPackagesAvailable = false;
        this.productInfoAvailable = false;
        this.sessionTimeoutException = false;
        this.dashboardProductInfoSubscriptionErrors = false;
        this.dashboardRpmInfoSubscriptionErrors = false;
        this.gettingInstalledHotfixes = true;
        this.foundInstalledHotfixes = false;
        this.getInstalledHotfixesSubscriptionErrors = false;
        this.gettingHotfixChangelog = false;
        this.foundHotfixChangelog = false;
        this.showChangelog = false;
        this.showHotfixTable = true;
        this.productName = this.getProductName();
    }
    DashboardComponent.prototype.ngOnInit = function () {
        if (this.emiService.getSessionTimeOutStatus()) {
            this.sessionTimeoutException = true;
            this.dashboardProductInfoSubscriptionErrors = true;
            this.dashboardRpmInfoSubscriptionErrors = true;
        }
        else {
            this.getProductInfo();
            this.getInstalledHotfixes();
            this.getRpmPackagesInfo();
        }
    };
    DashboardComponent.prototype.getProductName = function () {
        return this.emiService.getProductName();
    };
    DashboardComponent.prototype.getCompanyName = function () {
        return this.emiService.getCompanyName();
    };
    DashboardComponent.prototype.getRpmPackagesInfo = function () {
        var _this = this;
        this.dashboardService.getRpmPackages()
            .subscribe(function (rpmpackages) {
            _this.rpmpackages = rpmpackages;
            _this.rpmPackagesAvailable = true;
        }, function (error) {
            _this.errorMessage = error;
            console.error('DashboardComponent ==> getRpmPackagesInfo() ==> ' + _this.errorMessage);
            if (_this.errorMessage === _this.emiService.getSessionIdTimeoutError()) {
                _this.sessionTimeoutException = true;
                _this.emiService.setSessionTimedOut();
            }
            _this.dashboardProductInfoSubscriptionErrors = true;
        });
    };
    DashboardComponent.prototype.getProductInfo = function () {
        var _this = this;
        this.dashboardService.getProductInfo()
            .subscribe(function (productinfo) {
            _this.productinfo = productinfo;
            _this.productInfoAvailable = true;
        }, function (error) {
            _this.errorMessage = error;
            console.error('DashboardComponent ==> getProductInfo() ==> ' + _this.errorMessage);
            if (_this.errorMessage === _this.emiService.getSessionIdTimeoutError()) {
                _this.sessionTimeoutException = true;
                _this.emiService.setSessionTimedOut();
            }
            _this.dashboardRpmInfoSubscriptionErrors = true;
        });
    };
    DashboardComponent.prototype.resetHotfixLayout = function () {
        this.showChangelog = false;
        this.showHotfixTable = true;
    };
    DashboardComponent.prototype.getHotfixChangelog = function (hotfix) {
        var _this = this;
        console.log('HotfixComponent ==> getHotfixChangelog() ==> ' + hotfix);
        this.showHotfixTable = false;
        this.showChangelog = true;
        this.changelogHotfix = hotfix;
        this.gettingHotfixChangelog = true;
        this.dashboardService.getHotfixChangelog(hotfix)
            .subscribe(function (hotfixchangelog) {
            if (Array.isArray(hotfixchangelog) && hotfixchangelog.length > 0) {
                _this.foundHotfixChangelog = true;
                _this.changelogList = hotfixchangelog;
            }
            _this.gettingHotfixChangelog = false;
        }, function (error) {
            _this.errorMessage = error;
            console.error('HotfixComponent ==> getHotfixChangelog() ==> ' + _this.errorMessage);
            if (_this.errorMessage === _this.emiService.getSessionIdTimeoutError()) {
                _this.sessionTimeoutException = true;
                _this.emiService.setSessionTimedOut();
            }
            _this.getInstalledHotfixesSubscriptionErrors = true;
        });
    };
    DashboardComponent.prototype.getInstalledHotfixes = function () {
        var _this = this;
        this.dashboardService.getHotfixes()
            .subscribe(function (installedhotfixes) {
            _this.gettingInstalledHotfixes = false;
            if (Array.isArray(installedhotfixes) && installedhotfixes.length > 0) {
                _this.foundInstalledHotfixes = true;
                _this.hotfixes = installedhotfixes;
            }
        }, function (error) {
            _this.errorMessage = error;
            console.error('HotfixComponent ==> getInstalledHotfixes() ==> ' + _this.errorMessage);
            if (_this.errorMessage === _this.emiService.getSessionIdTimeoutError()) {
                _this.sessionTimeoutException = true;
                _this.emiService.setSessionTimedOut();
            }
            _this.getInstalledHotfixesSubscriptionErrors = true;
        });
    };
    DashboardComponent = __decorate([
        core_1.Component({
            moduleId: module.id,
            selector: 'system-dashboard',
            templateUrl: '../views/dashboard.component.html'
        }), 
        __metadata('design:paramtypes', [dashboard_service_1.DashboardService, emi_service_1.EMIService])
    ], DashboardComponent);
    return DashboardComponent;
}());
exports.DashboardComponent = DashboardComponent;
